# src/astroviper/distributed/__init__.py
from __future__ import annotations
from typing import TYPE_CHECKING

__all__ = [
    # submodules (so users can do astroviper.distributed.imaging)
    "imaging",
    "flagging",
    "calibration",
    "image_analysis",
    "visibility_manipulation",
    "utils",
]
